unit UntRss;

interface

uses sysutils, XMLIntf, comctrls, XMLDoc, JvComCtrls, XMLDom, stdctrls, dmMod, strutils,
  dialogs;


type
  PRSSFeedData = ^TRSSFeedData;
  TRSSFeedData = record
    URL : String;
    Description : WideString;
  end;

  function PopulateTree(HFileName : TFileName; FileSize: Integer; Url: string;
   indeximmagine : integer; TreeName : TJVTreeView; DocumentoXML : TXMLDocument) : String;
  procedure UpdateStatusPanel(pannello : Tstatuspanel; messaggio : string);
  procedure UpdateLabel(etichetta: TLabel; messaggio : string);
  procedure ClearTree(TreeName : TJVTreeView);
  function SanitazeDescription(testo : UTF8String): WideString;

implementation

function SanitazeDescription(testo : UTF8String): WideString;
begin
  testo := AnsiReplaceStr(testo,#146,'''');
  testo := AnsiReplaceStr(testo,#$93,'"');
  testo := AnsiReplaceStr(testo,#$94,'"');
  testo := AnsiReplaceStr(testo,#$A,'');
  testo := AnsiReplaceStr(testo, '<br>', #13#10);
  testo := AnsiReplaceStr(testo, '<b>','');
  testo := AnsiReplaceStr(testo, '</b>','');
  testo := AnsiReplaceStr(testo, '', '');
  testo := AnsiReplaceStr(testo, 'ì', '');
  testo := AnsiReplaceStr(testo, 'ù', '');
  testo := AnsiReplacestr(testo, '', '');

  result := testo;
end;

procedure ClearTree(TreeName : TJVTreeView);
var
  cnt : integer;
  rssFeedData : PRSSFeedData;
begin
  for cnt := 0 to TreeName.Items.Count - 1 do
  begin
    rssFeedData := PRSSFeedData(TreeName.Items[cnt].Data);
    if assigned(rssFeedData) then
    begin
      rssFeedData.URL := '';
      rssFeedData.Description := '';
    end;
    Dispose(TreeName.Items[cnt].Data);
  end;
  TreeName.Items.Clear;
end;

procedure UpdateStatusPanel(pannello : Tstatuspanel; messaggio : string);
begin
    pannello.text := messaggio;
end;

procedure UpdateLabel(etichetta: TLabel; messaggio : string);
begin
    etichetta.caption := messaggio;
end;

// PopulateTree: Ritorna True se ha successo, altrimenti ritorna la
// stringa di errore
function PopulateTree(HFileName : TFileName; FileSize: Integer;  Url: string;
indeximmagine : integer; TreeName : TJVTreeView; DocumentoXML : TXMLDocument) : String;
var
  nd : IXMLNode;
  rssRootData : PRSSFeedData;
  rootnode : TTreeNode;

  procedure ProcessItem();
  var
    rssFeedData : PRSSFeedData;
    tn : TTreeNode;
    title : string;
  begin

    New(rssFeedData);

    if Url = ModServ.URLOspedali then
      title := StringReplace(nd.ChildNodes.FindNode('title').Text,
      'segnalazione ospedale :','H.',[rfReplaceAll, rfIgnoreCase])
    else
      title := nd.ChildNodes.FindNode('title').Text;

    with nd.ChildNodes do
    begin
      rssFeedData.URL := FindNode('link').Text;
      rssFeedData.Description :=
        SanitazeDescription(UTF8Encode(FindNode('description').Text));
      //rssFeedData.Date  := FindNode('dc:date').Text;
    end;

    tn := TreeName.Items.AddChild(rootnode, title);
    tn.ImageIndex := indeximmagine;
    tn.Data := rssFeedData;

end;

begin
  New(rssRootData);

  DocumentoXML.FileName := HFileName;
  DocumentoXML.ParseOptions := [poResolveExternals, poValidateOnParse];

  DocumentoXML.Active := true;

try
  try

  //"chanel"
  nd := DocumentoXML.DocumentElement.ChildNodes.First;

  //title
  nd := nd.ChildNodes.First;

  //add top tree node
  rootnode := TreeName.Items.Add(nil,nd.Text);
  rootnode.ImageIndex := indeximmagine;

  rssRootData.URL := URL;
  rssRootData.Description := 'Dimensioni: ' + inttostr(FileSize);
  rootnode.Data := rssRootData;

  while nd.NodeName <> 'item' do
  begin
   nd := nd.NextSibling;
   if not assigned(nd) then
   begin
      Result := 'True';
      exit;
   end;
  end;

  rootnode.HasChildren := true;  // nominates as having children for expansion
  //loop through "items"
  while nd <> nil do
  begin
    ProcessItem();
    nd := nd.NextSibling;
  end;

  Result := 'True';

  except
  on E:EDOMParseError do
    begin
      Result := 'Invalid XML File';
      rootnode := TreeName.Items.Add(nil,Url);
      rootnode.ImageIndex := 4;
      rssRootData.URL := URL;
      rssRootData.Description := Result + ': ' + Url;
      rootnode.Data := rssRootData;
      Abort;
    end;
    else
    begin
      Result := 'Errore durante l''elaborazione del dato';
      rootnode := TreeName.Items.Add(nil,Url);
      rootnode.ImageIndex := 4;
      rssRootData.URL := URL;
      rssRootData.Description := Result + ': ' + Url;
      rootnode.Data := rssRootData;
      Abort;
    end;
  end;
finally
 //"close" RSS XML file
  if DocumentoXML.Active then
    DocumentoXML.Active := false;
 end;
end;


end.
